// simple.frag

#version 120
#extension GL_EXT_gpu_shader3 : enable

// fragment shader of phong shader

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int lightNumber;
uniform int lightRecieve[7];
uniform int useTexAlpha;

varying vec3 position;
varying vec3 normal;

float GetSpotEffect(int i, float spotAngleCos){
    float spotCosCutoff = gl_LightSource[i].spotCosCutoff;
    if (spotCosCutoff < -0.99){
	return 1.0;
    }
    
    float spotEffect = 0.0;
    float spotCosFade = cos(3.14159*gl_LightSource[i].spotCutoff/180 * 1.1);
    
    if (spotAngleCos > spotCosCutoff) {
    	spotEffect = 1.0;//pow(spotEffect, gl_LightSource[i].spotExponent);
    }
    else if (spotAngleCos > spotCosFade/*spotCosCutoff * 0.9*/) {
	float factor = (spotAngleCos - spotCosFade/*spotCosCutoff * 0.9*/) / (spotCosCutoff - spotCosFade);//spotCosCutoff / 0.1;
	spotEffect = -2.0*factor*factor*factor + 3.0*factor*factor;
    }
    else
	spotEffect = 0.0;

    return spotEffect;
}

void FogEffect(){
    if (fogMode == 0) return;

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(gl_Fog.color, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);
    
    vec3 fnormal = normalize(normal);
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }
    
    if (alpha > 1.0) alpha = 1.0;

    // ׂ͂ĔFF
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    vec3 viewVec = -normalize(position);
    
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	// fBt[Y 
	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }

    gl_FragColor = gl_FrontLightProduct[0].ambient + 
	gl_FrontMaterial.diffuse * diffuseTotal +
	gl_FrontMaterial.specular * specularTotal;
    gl_FragColor *= texColor;

    FogEffect();
    
    gl_FragColor[3] = alpha;
}
